﻿/*
 * Copyright 2010 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Collections.Generic;
using System.IO;

using NUnit.Framework;

namespace Vajhoej.Record.Test
{
    [TestFixture]
    public class TestUtil {
        private const int N = 10000;
        [Test]
        public void TestReadList()
        {
            byte[] b = { 0x02, 0x01, 0x00, 0x00,
                         0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                         0x41, 0x42, 0x43, 0x20, 0x20, 0x20, 0x20, 0x20 };
            byte[] bm = new byte[N*b.Length];
            for(int i = 0; i < N; i++)
            {
                Array.Copy(b, 0, bm, i*b.Length, b.Length);
            }
            try
            {
                IList<Data> lst = Util.ReadAll<Data>(typeof(Data), bm);
                Assert.AreEqual(N, lst.Count, "lst size");
                for(int i = 0; i < lst.Count; i++)
                {
                    Assert.AreEqual(258, lst[i].Iv, "iv #" + i);
                    Assert.AreEqual(123.456, lst[i].Xv, 0.0005, "xv #" + i);
                    Assert.AreEqual("ABC     ", lst[i].Sv, "sv #" + i);
                }
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
        [Test]
        public void TestWriteList()
        {
            byte[] b = { 0x02, 0x01, 0x00, 0x00,
                         0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                         0x41, 0x42, 0x43, 0x20, 0x20, 0x20, 0x20, 0x20 };
            try
            {
                IList<Data> lst = new List<Data>();
                for(int i = 0; i < N; i++)
                {
                    Data o = new Data();
                    o.Iv = 258;
                    o.Xv = 123.456;
                    o.Sv = "ABC     ";
                    lst.Add(o);
                }
                byte[] res = Util.WriteAll(typeof(Data), lst);
                Assert.AreEqual(N*b.Length, res.Length, "length");
                for(int i = 0; i < res.Length; i++)
                {
                    Assert.AreEqual(b[i % b.Length], res[i], "byte " + i);
                }
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
        [Test]
        public void TestReadStream()
        {
            byte[] b = { 0x02, 0x01, 0x00, 0x00,
                         0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                         0x41, 0x42, 0x43, 0x20, 0x20, 0x20, 0x20, 0x20 };
            byte[] bm = new byte[N*b.Length];
            for(int i = 0; i < N; i++)
            {
                Array.Copy(b, 0, bm, i*b.Length, b.Length);
            }
            MemoryStream mstm = new MemoryStream(bm);
            try
            {
                IList<Data> lst = new List<Data>();
                Util.ReadAll(typeof(Data), mstm, lst);
                Assert.AreEqual(N, lst.Count, "lst size");
                for(int i = 0; i < lst.Count; i++)
                {
                    Assert.AreEqual(258, lst[i].Iv, "iv #" + i);
                    Assert.AreEqual(123.456, lst[i].Xv, 0.0005, "xv #" + i);
                    Assert.AreEqual("ABC     ", lst[i].Sv, "sv #" + i);
                }
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
            catch (IOException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
        [Test]
        public void TestWriteStream()
        {
            byte[] b = { 0x02, 0x01, 0x00, 0x00,
                         0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                         0x41, 0x42, 0x43, 0x20, 0x20, 0x20, 0x20, 0x20 };
            try
            {
                IList<Data> lst = new List<Data>();
                for(int i = 0; i < N; i++)
                {
                    Data o = new Data();
                    o.Iv = 258;
                    o.Xv = 123.456;
                    o.Sv = "ABC     ";
                    lst.Add(o);
                }
                MemoryStream mstm = new MemoryStream();
                Util.WriteAll(typeof(Data), lst, mstm);
                byte[] res = mstm.ToArray();
                Assert.AreEqual(N*b.Length, res.Length, "length");
                for(int i = 0; i < res.Length; i++)
                {
                    Assert.AreEqual(b[i % b.Length], res[i], "byte " + i);
                }
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
            catch (IOException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
        [Test]
        public void TestReadHandler()
        {
            byte[] b = { 0x02, 0x01, 0x00, 0x00,
                         0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                         0x41, 0x42, 0x43, 0x20, 0x20, 0x20, 0x20, 0x20 };
            byte[] bm = new byte[N*b.Length];
            for(int i = 0; i < N; i++)
            {
                Array.Copy(b, 0, bm, i*b.Length, b.Length);
            }
            MemoryStream mstm = new MemoryStream(bm);
            try
            {
                Util.ReadAll<Data>(typeof(Data), mstm, delegate(Data o)
                                                 {
                                                     Assert.AreEqual(258, o.Iv, "iv");
                                                     Assert.AreEqual(123.456, o.Xv, 0.0005, "xv");
                                                     Assert.AreEqual("ABC     ", o.Sv, "sv");
                                                 });
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
            catch (IOException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
        [Test]
        public void TestCopyArrays()
        {
            byte[] b = { 0x02, 0x01, 0x00, 0x00,
                         0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                         0x41, 0x42, 0x43, 0x20, 0x20, 0x20, 0x20, 0x20 };
            byte[] b2 = { 0x03, 0x01, 0x00, 0x00,
                          0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                          0x41, 0x42, 0x43, 0x58, 0x58, 0x58, 0x58, 0x58 };
            try
            {
                byte[] res = Util.CopyAll<Data,Data>(typeof(Data), b, typeof(Data), delegate(Data o)
                                                                         { 
                                                                             Data o2 = new Data();
                                                                             o2.Iv = o.Iv + 1;
                                                                             o2.Xv = o.Xv;
                                                                             o2.Sv = o.Sv.Replace(' ', 'X');
                                                                             return o2;
                                                                         });
                Assert.AreEqual(b2.Length, res.Length, "length");
                for(int i = 0; i < res.Length; i++)
                {
                    Assert.AreEqual(b2[i], res[i], "byte " + i);
                }
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
#if DN35
        [Test]
        public void TestCopyArraysLambda()
        {
            byte[] b = { 0x02, 0x01, 0x00, 0x00,
                         0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                         0x41, 0x42, 0x43, 0x20, 0x20, 0x20, 0x20, 0x20 };
            byte[] b2 = { 0x03, 0x01, 0x00, 0x00,
                          0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                          0x41, 0x42, 0x43, 0x58, 0x58, 0x58, 0x58, 0x58 };
            try
            {
                byte[] res = Util.CopyAll<Data,Data>(typeof(Data), b, typeof(Data), o => new Data { Iv = o.Iv + 1, Xv = o.Xv, Sv = o.Sv.Replace(' ', 'X') });
                Assert.AreEqual(b2.Length, res.Length, "length");
                for(int i = 0; i < res.Length; i++)
                {
                    Assert.AreEqual(b2[i], res[i], "byte " + i);
                }
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
#endif
        [Test]
        public void TestCopyStreams()
        {
            byte[] b = { 0x02, 0x01, 0x00, 0x00,
                         0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                         0x41, 0x42, 0x43, 0x20, 0x20, 0x20, 0x20, 0x20 };
            byte[] b2 = { 0x03, 0x01, 0x00, 0x00,
                          0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                          0x41, 0x42, 0x43, 0x58, 0x58, 0x58, 0x58, 0x58 };
            try
            {
                MemoryStream inmstm = new MemoryStream(b);
                MemoryStream outmstm = new MemoryStream();
                Util.CopyAll<Data,Data>(typeof(Data),inmstm, typeof(Data), outmstm, delegate(Data o)
                                                                         { 
                                                                             Data o2 = new Data();
                                                                             o2.Iv = o.Iv + 1;
                                                                             o2.Xv = o.Xv;
                                                                             o2.Sv = o.Sv.Replace(' ', 'X');
                                                                             return o2;
                                                                         });
                byte[] res = outmstm.ToArray();
                Assert.AreEqual(b2.Length, res.Length, "length");
                for(int i = 0; i < res.Length; i++)
                {
                    Assert.AreEqual(b2[i], res[i], "byte " + i);
                }
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
            catch (IOException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
#if DN35
        [Test]
        public void TestCopyStreamsLambda()
        {
            byte[] b = { 0x02, 0x01, 0x00, 0x00,
                         0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                         0x41, 0x42, 0x43, 0x20, 0x20, 0x20, 0x20, 0x20 };
            byte[] b2 = { 0x03, 0x01, 0x00, 0x00,
                          0x77, (byte)0xBE, (byte)0x9F, 0x1A, 0x2F, (byte)0xDD, 0x5E, 0x40,
                          0x41, 0x42, 0x43, 0x58, 0x58, 0x58, 0x58, 0x58 };
            try
            {
                MemoryStream inmstm = new MemoryStream(b);
                MemoryStream outmstm = new MemoryStream();
                Util.CopyAll<Data,Data>(typeof(Data),inmstm, typeof(Data), outmstm, o => new Data { Iv = o.Iv + 1, Xv = o.Xv, Sv = o.Sv.Replace(' ', 'X') });
                byte[] res = outmstm.ToArray();
                Assert.AreEqual(b2.Length, res.Length, "length");
                for(int i = 0; i < res.Length; i++)
                {
                    Assert.AreEqual(b2[i], res[i], "byte " + i);
                }
            }
            catch (RecordException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
            catch (IOException e)
            {
                Assert.Fail("Unexpected exception: " + e);
            }
        }
#endif
    }
}